## 高阶组件

高阶组件是 `React` 中对组件逻辑进行重用的高级技术，但是在这里我们还是希望大家谨慎使用高阶组件。 

现阶段，我们只支持一种形式的高阶组件，就是在原组件上扩展属性，如下所示：

```javascript
function Hoc (WrappComponent) {
  
    return class Inner extends  React.HocComponnet{ // 继承HocComponnet
        
        componentDidMount() {
            
        }
        
        shouldComponentUpdate() {
            
        }
        
        render() {
            return React.createElement(
                     WrappComponent, 
                     {
                         ...this.props,  // 扩展其他属性
                         ...this.hocProps // 添加hocProps， 这个对象由 HocComponnet实现，开发者不用关心具体实现
                     } 
                )  
        }
    }
}

``` 

> 这里有几点需要注意：

* 类必须继承 `React.HocComponent` ，`HocComponent` 只存在与微信小程序平台
* 不允许使用``JSX``语法， 创建元素使用 `React.createElement`
* 属性最后一定要添加 `this.hocProps`

由于以上的限制存在，通常情况下对于 Hoc.js 这个高阶组件， 需要存在两个文件 Hoc.js 用于 RN， Hoc.wx.js 用于小程序


以下是小程序版本react-redux对connext的实现（源代码在`packages/wx-react-redux`）：

```javascript
export default function connect(mapStateToProps, mapDispatchToProps) {

    return function (WrappedComponent) {
        return class Hoc extends HocComponent {
            static contextTypes = {
                store: PropTypes.object
            }

            constructor(props, context) {
                super(props, context)

                this.store = props.store || context.store

                this.unsubscribe = this.store.subscribe(() => {
                    this.setState({})
                })

                this.memorizeProps = this.calculateProps()

            }

            calculateProps() {
                let o1 = null
                if (mapStateToProps) {
                    o1 = mapStateToProps(this.store.getState(), this.props)
                }


                let o2 = null
                if (mapDispatchToProps) {
                    o2 = mapDispatchToProps(this.store.dispatch, this.props)
                } else {
                    o2 = {
                        dispatch: this.store.dispatch
                    }
                }

                return {
                    ...o1,
                    ...o2
                }
            }

            shouldComponentUpdate() {
                const nextProps = this.calculateProps()

                const isEqual = shallowEqual(nextProps, this.memorizeProps)
                if (isEqual) {
                    return false
                } else {
                    this.memorizeProps = nextProps
                    return true
                }
            }

            componentWillUnmount() {
                this.unsubscribe()
                this.unsubscribe = null

            }

            render() {
                return (
                    React.createElement(
                        WrappedComponent,
                        {
                            ...this.props,
                            ...this.memorizeProps,
                            ...this.hocProps
                        }
                    )
                )
            }
        }

    }
}
```
